function LootSquid:RegisterEvents()

    -- hook chat events
    LootSquid:RegisterEvent("CHAT_MSG_BN_WHISPER", "OnChatMessage")
    LootSquid:RegisterEvent("CHAT_MSG_RAID", "OnChatMessage")
    LootSquid:RegisterEvent("CHAT_MSG_WHISPER", "OnChatMessage")
    LootSquid:RegisterEvent("CHAT_MSG_GUILD", "OnChatMessage")
    LootSquid:RegisterEvent("CHAT_MSG_PARTY", "OnChatMessage")

    -- hook loot events
    LootSquid:RegisterEvent("LOOT_OPENED", "OnLootOpened")
    LootSquid:RegisterEvent("LOOT_CLOSED", "OnLootClosed")
    LootSquid:RegisterEvent("LOOT_SLOT_CHANGED", "OnLootSlotChanged")
    LootSquid:RegisterEvent("LOOT_SLOT_CLEARED", "OnLootSlotCleared")
end

function LootSquid:OnLootSlotChanged(a, b, c, d, e, f, g, h)
--    LootSquid:Print("OnLootSlotChanged: "
--            ..tostring(a)..", "..tostring(b)..", "..tostring(c)..", "..tostring(d)..", "
--            ..tostring(e)..", "..tostring(f)..", "..tostring(g)..", "..tostring(h)..", ")
end

--[[
-- Fires when the contents of a loot slot are removed.
-- Can fire due to the player looting the slot's contents
-- or due to them being taken by another group member.
 ]]
function LootSquid:OnLootSlotCleared(event, slotId)
    -- slotId: The numeric id of the slot that was looted.
    LootSquid:UpdateLootTable(false)
end

function LootSquid:OnChatMessage(event, message, sender, language, channelString, target, flags,
unknown, channelNumber, channelName, unknown, counter, guid)

    local method, partyMaster, raidMaster = GetLootMethod()
    -- do not process chat message unless master looter
    if not (method == "master" and partyMaster == 0) then
        return
    end


    -- if loot dist in session...

    local isNeed = LootSquid:strStartsWith(message, "!need")
    local isGreed = LootSquid:strStartsWith(message, "!greed")

    if (_G["LootSquidSession"].currentLootSlot < 1 and (isNeed or isGreed)) then
        SendChatMessage("LootSquid: The loot master does not currently have an active loot session",
            "WHISPER",
            nil,
            sender)
        return
    end


    if (isNeed) then

        _G["LootSquidSession"].playersNeeding[sender] = true
        self:UpdateRequestFrame()

        SendChatMessage("LootSquid: Your need request have been received. Please wait.",
            "WHISPER",
            nil,
            sender)
    end

    if (isGreed) then

        _G["LootSquidSession"].playersGreeding[sender] = true
        self:UpdateRequestFrame()

        SendChatMessage("LootSquid: Your greed request have been received. Please wait.",
            "WHISPER",
            nil,
            sender)
    end
end

function LootSquid:UpdateLootTable(doOpenGui)
    for k in pairs(_G["LootSquidSession"].loot) do
        _G["LootSquidSession"].loot[k] = nil
    end


    local method, partyMaster, raidMaster = GetLootMethod()
    local lootThreshold = GetLootThreshold()
    local playerName, playerRealm = UnitName("player")


    -- if player is the master looter
    if (_G["LootSquidDebug"].alwaysParseLoot == true
            or (method == "master" and partyMaster == 0)) then

        local lootSlots = GetNumLootItems()
        local lootSlotsAtleastThreshold = 0

        _G["LootSquidSession"].lootSlots = lootSlots

        for lootSlot = 1, lootSlots do

            --[[
            -- The lootSlot will remain concistent for items as long as the loot box remains open.
            -- If the loot box is closed after some items have been looted, lootSlot will have changed
            -- for items which originally was placed after a looted item.
            -- ]]
            local itemTexture, itemName, itemQuantity, itemQuality, itemLocked = GetLootSlotInfo(lootSlot)

            if itemQuality and itemQuality >= lootThreshold then
                lootSlotsAtleastThreshold = lootSlotsAtleastThreshold + 1
            end

            local itemLootType = GetLootSlotType(lootSlot)


--            self:Print("GetLootSlotInfo "
--                    .. tostring(lootSlot) .. ", "
--                    .. tostring(itemName) .. ", "
--                    .. tostring(itemQuantity) .. ", "
--                    .. tostring(itemQuality) .. ", "
--                    .. tostring(itemLocked) .. ", "
--                    .. tostring(itemLootType)
--            )

            local itemLink = GetLootSlotLink(lootSlot)

            _G["LootSquidSession"].loot[lootSlot] = {
                itemName = itemName,
                itemQuantity = itemQuantity,
                itemQuality = itemQuality,
                itemLocked = itemLocked,
                itemLootType = itemLootType,
                itemLink = itemLink,
                candidates = {}
            }

            for candidateIndex = 1, 40 do

                local candidateName = GetMasterLootCandidate(lootSlot, candidateIndex)
                if candidateName then

                    _G["LootSquidSession"].loot[lootSlot].candidates[candidateName] = {
                        candidateIndex = candidateIndex,
                    }

                else
                    --LootSquid:Print("no candidate name")
                end
            end
        end

        if (
            (_G["LootSquidDebug"].alwaysParseLoot and doOpenGui)
            or
                (
                    lootSlotsAtleastThreshold > 0
                        and doOpenGui
                        and lootSlots > 0
                        and (not InCombatLockdown() or IsShiftKeyDown())
                )
            ) then
            self:OpenLootGui()
        else
            self:OnUpdateLootTable()
        end
    end
end

function LootSquid:OnLootOpened(autoLoot)
    self:UpdateLootTable(true)
end

function LootSquid:OnLootClosed()
end
